{-
 *  Programmer:	Piotr Borek
 *  E-mail:     piotrborek@op.pl
 *  Copyright 2017 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-}
{-# LANGUAGE QuasiQuotes #-}

module Mp.UI.MainStatusBar (
    mainStatusBarNew
) where

import           Control.Monad.Catch
import           Simple.Locale.TranslateTH
import           Simple.UI.All

import           Mp.Locale.TranslateFile   ()
import           Mp.Player.Client
import           Mp.UI.MpData
import           Mp.Utils.Utils

mainStatusBarNew :: UIApp MpData StatusBar
mainStatusBarNew = do
    statusBar <- statusBarNew
    set statusBar textLeft $ Just "Left"
    set statusBar textRight $ Just "Right"

    mpData <- view appUserData
    set statusBar colorForeground (mpData ^. mpColors . statusForeground)
    set statusBar colorBackground (mpData ^. mpColors . statusBackground)
    set statusBar colorStyle DrawStyleBold

    scheduleStatusFromServer statusBar
    return statusBar
  where
    statusToString status = case status of
        "Stopped" -> [tr|Stopped|]
        "Playing" -> [tr|Playing|]
        "Paused"  -> [tr|Paused|]
        _         -> [tr|Unknown|]

    scheduleStatusFromServer statusBar = mainScheduleRepeat 200 $
        catchAll
            (do
                (status, dur, pos) <- clientSendGetStatus
                flags <- clientSendGetFlags
                vol   <- clientSendVolumeGet
                set statusBar textLeft $ Just ("  " ++ statusToString status ++ " [" ++ timeToString pos ++ "/" ++ timeToString dur ++ "]")
                set statusBar textRight $ Just ("[" ++ flags ++ "]  " ++ [tr|Volume|] ++ ": " ++ vol ++ " %  ")
            )
            (\_ -> do
                set statusBar textLeft $ Just [tr|Connection error|]
                set statusBar textRight Nothing
            )
